'use strict';

var Hjson = require('hjson');
const fs = require('fs');
const Joi = require('joi');
const { exit } = require('process');

// cat config.dist.json.4.joi.schema.validator.json | joi-machine
const configSchema = Joi.object().keys({
    license: Joi.object().keys({
        file: Joi.binary(), 
        url: Joi.binary(), 
        templicensefolder: Joi.binary(), 
        autorenew: Joi.boolean(),
        autorenewdays: Joi.number().integer()
        }), 
    logger: Joi.object().keys({nodejs: Joi.object().keys({path: Joi.string(), logrequest: Joi.boolean(), logresponse: Joi.boolean()}), 'native': Joi.object().keys({path: Joi.string(), filename: Joi.string(), rolloutwhenrunning: Joi.number().integer()}), printer: Joi.object().keys({path: Joi.string(), level: Joi.string(), levelcustom: Joi.number().integer(), filename: Joi.string()}), extras: Joi.object().keys({logmetrics: Joi.boolean(), logmetricsevery: Joi.number().integer()}), fulldebug: Joi.boolean()}), 
    remoteconfig: Joi.object().keys({
        active: Joi.boolean(), 
        url: Joi.string().uri()}), 
    printing: Joi.object().keys({
        transaction: Joi.object().keys({enabled: Joi.boolean()})}), 
    dlq: Joi.object().keys({
        enabled: Joi.boolean().required(),
        directory: Joi.string().required()}),
    fiscalmemory: Joi.object().keys({
        cache: Joi.object().keys({
            directory: Joi.string().required()}),
        }),
    posnet: Joi.object().keys({
        driver: Joi.string(), commandresponsetimeout: Joi.number().integer(), commanddelay: Joi.number().integer(), waitdelay: Joi.number().integer(), responsewaitdelay: Joi.number().integer(), 
        ping: Joi.object().keys({
            enabled: Joi.boolean().required(), 
            interval: Joi.number().integer().required(),
            type: Joi.string().required().valid('status', 'date')
        }), 
        edocument: Joi.object().keys({
            service: Joi.object().keys({
                url: Joi.string().required(), 
                sha1_footprint_update: Joi.object().keys({
                    enabled: Joi.boolean().required(),
                    interval: Joi.number().integer().required(),
                    onstart: Joi.boolean().required(),
                })
            }).required()
        }),
        connection: Joi.object().keys({type: Joi.string(), rs232: Joi.object().keys({disable_ioctl: Joi.boolean(), params: Joi.string()}), eth: Joi.object().keys({ip: Joi.string(), port: Joi.string()}), tunnel: Joi.object().keys({url: Joi.string().uri()})})}), 
    throttling: Joi.object().keys({
        rate: Joi.string(), 
        rate_limit: Joi.number().integer(), 
        disabled: Joi.boolean(), 
        privileged: Joi.object().keys({ip: Joi.array().items(Joi.string())})}), 
    retry: Joi.object().keys({
        callbacks: Joi.object().keys({times: Joi.number().integer(), interval: Joi.number().integer()}), 
        errors: Joi.array().items(Joi.object().keys({error: Joi.number().integer(), active: Joi.boolean(), times: Joi.number().integer(), interval: Joi.number().integer()}))}), 
    plugins: Joi.object().keys({
        'azure-service-bus': Joi.object().keys({
            enabled: Joi.boolean(), 
            reinitOnError: Joi.boolean(), 
            reinitDelay: Joi.number().integer(), 
            service: Joi.object().keys({active: Joi.boolean(), connectionstring: Joi.string(), queuename: Joi.string(), autoack: Joi.boolean()}), 
            input: Joi.object().keys({connectionstring: Joi.string(), queuename: Joi.string(), autoack: Joi.boolean()}), 
            output: Joi.object().keys({active: Joi.boolean(), connectionstring: Joi.string(), queuename: Joi.string()})
        }), 
        'amqp-client': Joi.object().keys({
            enabled: Joi.boolean(), 
            input: Joi.object().keys({amqp_url: Joi.string().uri(), connectionparams: Joi.string(), queuename: Joi.string(), queue_args: Joi.object().keys({durable: Joi.boolean()}), requeue: Joi.boolean()}), 
            output: Joi.object().keys({
                active: Joi.boolean(), 
                amqp_url: Joi.string().uri(), 
                connectionparams: Joi.string(), 
                exchange: Joi.string(), 
                exchangetype: Joi.string(), 
                exchange_args: Joi.object().keys({durable: Joi.boolean()}), routingkey: Joi.binary()
            })
        }),
        'xml-custom-input': Joi.object().keys({
            enabled: Joi.boolean()
        })
    }),
    token: Joi.object().keys({
        active: Joi.boolean(), 
        secret: Joi.string(), 
        authservice: Joi.object().keys({url: Joi.string().uri()})}), 
    heartbeat: Joi.object().keys({
        allowed: Joi.array().items(Joi.string())}), 
    server: Joi.object().keys({
        http: Joi.object().keys({active: Joi.boolean(), host: Joi.string(), port: Joi.number().integer()}), 
        https: Joi.object().keys({active: Joi.boolean(), host: Joi.string(), port: Joi.number().integer(), sslcertificates: Joi.object().keys({key: Joi.string(), crt: Joi.string()})})})
});


// -----------------------------------------------------------------------------------
// UWAGA!
// Od wersji 2.5 wszystkie zmiany konfiguracyjne nalezy wprowadzac w pliku config.json
// -----------------------------------------------------------------------------------
console.log('Prepare to read config.json');
let rawdata = fs.readFileSync('config.json');
const config = Hjson.parse(rawdata.toString('utf8'));

var test = configSchema.validate(config);
//console.log(test);
if (test.error != null) {
    console.log('Configuration validation error:' + JSON.stringify(test));
    exit(1);
}

config['buildversion'] = '4.3';
config['schemaversion'] = '4.1';
console.log('Config successfully loaded from file config.json');
console.log('Build version: ' + config['buildversion']);
console.log('Schema version: ' + config['schemaversion']);

module.exports = config;
