#!/bin/bash
if ! command -v stty &> /dev/null
then
   echo "+--------------------------------------------------------------------+"
   echo "| !! ERROR !!                                                        |"
   echo "| stty could not be found                                            |"
   echo "+--------------------------------------------------------------------+"
   exit
fi

rs232_params=$1
if [ -z $rs232_params ]; then
   echo "Sample use: $0 '/dev/ttyS0,9600,8,N,1,N'"
   exit 1
fi

echo "###> Parsing '$rs232_params'"
IFS=', ' read -r -a array <<< "$rs232_params"
serialport="${array[0]}"
brate="${array[1]}"
cbits="${array[2]}"
cpar="${array[3]}"
bstop="${array[4]}"
flowctrl="${array[5]}"
         
CONTROL_SETTINGS=""
INPUT_SETTINGS="-ignbrk -brkint -parmrk -istrip -inlcr -igncr -icrnl -ixon -ixoff -iuclc -ixany -imaxbel -iutf8"
OUTPUT_SETTINGS="-ocrnl -ofdel -ofill -olcuc -onlcr -onlret -onocr -opost tab0 vt0 nl0 cr0 ff0 bs0 nl0"
LOCAL_SETTINGS="-echo -echoe -echok -echonl -echoprt -echoctl -echoke -icanon -iexten -isig -noflsh -tostop -xcase"
if [ "$cpar" = "N" ] || [ "$cpar" = "n" ];then
   CONTROL_SETTINGS="$CONTROL_SETTINGS -parenb -parodd"
   INPUT_SETTINGS="$INPUT_SETTINGS ignpar -inpck"
fi
if [ "$cpar" = "E" ] || [ "$cpar" = "e" ];then
   CONTROL_SETTINGS="$CONTROL_SETTINGS parenb -parodd"
   INPUT_SETTINGS="$INPUT_SETTINGS -ignpar inpck"
fi
if [ "$cpar" = "O" ] || [ "$cpar" = "o" ];then
   CONTROL_SETTINGS="$CONTROL_SETTINGS parenb parodd"
   INPUT_SETTINGS="$INPUT_SETTINGS -ignpar inpck"
fi

if [ "$bstop" = "1" ];then
   CONTROL_SETTINGS="$CONTROL_SETTINGS -cstopb"
fi
if [ "$bstop" = "2" ];then
   CONTROL_SETTINGS="$CONTROL_SETTINGS cstopb"
fi

if [ "$crtscts" = "N" ] || [ "$crtscts" = "n" ];then
   CONTROL_SETTINGS="$CONTROL_SETTINGS -crtscts"
fi
if [ "$crtscts" = "Y" ] || [ "$crtscts" = "y" ];then
   CONTROL_SETTINGS="$CONTROL_SETTINGS crtscts"
fi
         
echo "###> Setting $serialport params [$brate,$cbits,$cpar,$bstop,$flowctrl]....."
stty -F $serialport $brate $CONTROL_SETTINGS cs$cbits cread clocal $INPUT_SETTINGS $OUTPUT_SETTINGS $LOCAL_SETTINGS
echo "###> $serialport configuration done"

echo "###> Preparing to send sample data to $serialport"
exec 3<$serialport                        #REDIRECT SERIAL OUTPUT TO FD 3
cat <&3 > /tmp/ttyDump.dat &              #REDIRECT SERIAL OUTPUT TO FILE
PID=$!                                    #SAVE PID TO KILL CAT
echo "###> Sending TermInfo (2 50 48 28 49 3 44) to $serialport"
echo "###> Expected response is a transaction status from the terminal"
echo -e "\x02\x32\x30\x1C\x31\x03\x2C" > $serialport              #SEND COMMAND STRING TO SERIAL PORT
sleep 2s                                #WAIT FOR RESPONSE
kill $PID 2>/dev/null                     #KILL CAT PROCESS
echo "###> Waiting"
wait $PID 2>/dev/null                     #SUPRESS "Terminated" output

exec 3<&-                                 #FREE FD 3
stat --printf="###> Response size: %s bytes\n" /tmp/ttyDump.dat
cat /tmp/ttyDump.dat                      #DUMP CAPTURED DATA

echo "###> Done"
