#!/bin/bash
if [ ! -d "./node_modules" ]; then
   echo "+------------------------------------------------------------------------+"
   echo "| !! WARNING !!                                                          |"
   echo "| node_modules doesn't exist                                             |"
   echo "| did you forget 'npm install' ?                                         |"
   echo "+------------------------------------------------------------------------+"
fi

if ! command -v stty &> /dev/null
then
   echo "+------------------------------------------------------------------------+"
   echo "| !! ERROR !!                                                            |"
   echo "| stty could not be found                                                |"
   echo "+------------------------------------------------------------------------+"
   exit
fi

if ! command -v yq &> /dev/null
then
   echo "+------------------------------------------------------------------------+"
   echo "| !! WARNING !!                                                          |"
   echo "| yq command not found. Automatic pre-validation of config.json file     |"
   echo "| and environment preparation cannot be performed. It's strongly         |"
   echo "| suggested to install yq (apt-get install yq) or (yum install yq)       |"
   echo "| --- or ---                                                             |"
   echo "| sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys CC86BB64 |"
   echo "| sudo apt-get install software-properties-common                        |"
   echo "| sudo add-apt-repository ppa:rmescandon/yq                              |"
   echo "| sudo apt update                                                        |"
   echo "| sudo apt install yq                                                    |"
   echo "| --- or ---                                                             |"
   echo "| sudo apt update                                                        |"
   echo "| sudo apt install snapd                                                 |"
   echo "| sudo reboot                                                            |"
   echo "| sudo snap install yq                                                   |"
   echo "+------------------------------------------------------------------------+"
else

   #ingenico_protocol=`cat config.yml | yq r - 'default.ingenico.protocol'`
   ingenico_connection_type=`cat config.yml | yq -r '.default.ingenico.connection.type'`
   ingenico_connection_rs232_params=`cat config.yml | yq -r '.default.ingenico.connection.rs232.params'`

   if [ "$ingenico_connection_type" = "rs232" ]
   then
      echo "+--------------------------------------------------------------------+"
      echo "| !! INFORMATION !!                                                  |"
      echo "| you are about to run ingenicoserver on ARM arch or x86/x64 arch    |"
      echo "| It's important to properly configure your serial port before       |"
      echo "| launching IngenicoServer                                           |"
      echo "+--------------------------------------------------------------------+"
     
      IFS=', ' read -r -a array <<< "$ingenico_connection_rs232_params"
      serialport="${array[0]}"
      brate="${array[1]}"
      cbits="${array[2]}"
      cpar="${array[3]}"
      bstop="${array[4]}"
      flowctrl="${array[5]}"
      
      CONTROL_SETTINGS=""
      INPUT_SETTINGS="-ignbrk -brkint -parmrk -istrip -inlcr -igncr -icrnl -ixon -ixoff -iuclc -ixany -imaxbel -iutf8"
      OUTPUT_SETTINGS="-ocrnl -ofdel -ofill -olcuc -onlcr -onlret -onocr -opost tab0 vt0 nl0 cr0 ff0 bs0 nl0"
      LOCAL_SETTINGS="-echo -echoe -echok -echonl -echoprt -echoctl -echoke -icanon -iexten -isig -noflsh -tostop -xcase"
      if [ "$cpar" = "N" ] || [ "$cpar" = "n" ];then
         CONTROL_SETTINGS="$CONTROL_SETTINGS -parenb -parodd"
         INPUT_SETTINGS="$INPUT_SETTINGS ignpar -inpck"
      fi
      if [ "$cpar" = "E" ] || [ "$cpar" = "e" ];then
         CONTROL_SETTINGS="$CONTROL_SETTINGS parenb -parodd"
         INPUT_SETTINGS="$INPUT_SETTINGS -ignpar inpck"
      fi
      if [ "$cpar" = "O" ] || [ "$cpar" = "o" ];then
         CONTROL_SETTINGS="$CONTROL_SETTINGS parenb parodd"
         INPUT_SETTINGS="$INPUT_SETTINGS -ignpar inpck"
      fi
      
      if [ "$bstop" = "1" ];then
         CONTROL_SETTINGS="$CONTROL_SETTINGS -cstopb"
      fi
      if [ "$bstop" = "2" ];then
         CONTROL_SETTINGS="$CONTROL_SETTINGS cstopb"
      fi
      
      if [ "$crtscts" = "N" ] || [ "$crtscts" = "n" ];then
         CONTROL_SETTINGS="$CONTROL_SETTINGS -crtscts"
      fi
      if [ "$crtscts" = "Y" ] || [ "$crtscts" = "y" ];then
         CONTROL_SETTINGS="$CONTROL_SETTINGS crtscts"
      fi
      
      echo "Setting $serialport params [$brate,$cbits,$cpar,$bstop,$flowctrl]....."
      stty -F $serialport $brate $CONTROL_SETTINGS cs$cbits cread clocal $INPUT_SETTINGS $OUTPUT_SETTINGS $LOCAL_SETTINGS
      echo "Setting serial port params is done"

   fi
fi

export INGENICO_LIB_SHARE_DIR=`pwd`/
export LD_LIBRARY_PATH=`pwd`
npm start